#! /usr/bin/perl

## Aufgabe von CleanupMSYS.pl ist es, den VxMail-Folder des Mail-System-Admins
## ein wenig zu verkleinern.
## Dazu werden alle E-Mails, die lter als eine bestimmte Uhrzeit sind, aus
## dem Folder entfernt.

use Cwd;
use Socket;
use IO::Handle;

sub pop3_answer (*);
sub pop3_result (*);
sub pop3_ts (*);
sub pop3_quit (*$);
sub usage (@);

($progname = $0) =~ s(.*/)();
($progpath = $0) =~ s(\Q$progname\E$)();
$progpath =~ s(^[.]/)(cwd () . "/$1")e;
$progpath =~ s(^([^./])|^$)(cwd () . "/$1")e;
$progpath =~ s(/$)();
$filler = ' ' x (length $progname);

$silent = 0;
$port = undef;
while (@ARGV > 0 && ($_ = $ARGV[0]) =~ /^-/) {
    shift;
    last if (/^--$/);
    if (/^-h$|^-help$|^--help$/) {
	usage ();
    } elsif (/^-q$|^--quiet$|^-s$|^--silent$/) {
	$silent = 1;
    } elsif (/^-p(.*)|^--port=(.+)|^--port$/) {
	usage ("missing argument for option `$_'!") unless ($1 or @ARGV);
	$port = (($1) ? ($1) : (shift));
    } elsif (/^-o(.*)|^--outfile=(.+)|^--outfile$/) {
	usage ("missing argument for option `$_'!") unless ($1 or @ARGV);
	$outfile = (($1) ? ($1) : (shift));
    } else {
	usage ("unknown option `$_'!");
    }
}

unless (@ARGV >= 3) { usage ("missing arguments"); }

$server = (shift);
$cmd = (shift);
$user = (shift);

unless (defined $port) {
    $port = getservbyname ('pop3', 'tcp') or
	die "$progname: invalid service id!\n";
}
$proto = getprotobyname ('tcp');
$iaddr = inet_aton ($server);
$paddr = sockaddr_in ($port, $iaddr);

open OUT, ($outfile ? ">$outfile" : ">&STDOUT");
OUT->autoflush (1);
socket (SOCK, PF_INET, SOCK_STREAM, $proto) || die "$progname(socket): $!\n";
connect (SOCK, $paddr) || die "$progname (connect): $!\n";
SOCK->autoflush (1);

$md5ts = pop3_ts (SOCK);
die "$progname(MD5TS): $!\n" unless (defined $md5ts);
$pass = pwdprompt ();
if ($md5ts) {
    $md5dig = md5digest ("$md5ts$pass");
    print SOCK "APOP $user $md5dig\r\n";
    if (pop3_answer (SOCK)) { goto APOP_OK; }
}
print SOCK "USER $user\r\n";
unless (pop3_answer (SOCK)) { pop_quit (SOCK, "invalid user ::MAILSYS::"); }
print SOCK "PASS $pass\r\n";
unless (pop3_answer (SOCK)) { pop_quit (SOCK, "invalid password"); }

APOP_OK:

if ($cmd eq 'list' or $cmd eq 'uidl') {
    if (@ARGV) { $arg = (shift); $arg = " $arg"; }
    print SOCK "$cmd$arg\r\n";
    $answer = pop3_result (SOCK);
    pop_quit(SOCK, "protocol error!\n") unless (defined $answer);
    if ($answer) {
	if ($arg) {
	    print OUT "$answer\n";
	} else {
	    while (<SOCK>) { last if ($_ eq ".\r\n"); print OUT $_; }
	}
    }
} elsif ($cmd eq 'retr') {
    usage ("retr requires an additional argument") unless (@ARGV);
    $arg = (shift);
    print SOCK "$cmd $arg\r\n";
    $answer = pop3_answer (SOCK);
    pop_quit(SOCK, "protocol error!\n") unless (defined $answer);
    if ($answer) { while (<SOCK>) { last if ($_ eq ".\r\n"); print OUT $_; } }
} elsif ($cmd eq 'top') {
    usage ("top requires two additional arguments") unless (@ARGV >= 2);
    $arg = (shift); $arg1 = (shift);
    print SOCK "$cmd $arg $arg1\r\n";
    $answer = pop3_answer (SOCK);
    pop_quit(SOCK, "protocol error!\n") unless (defined $answer);
    if ($answer) { while (<SOCK>) { last if ($_ eq ".\r\n"); print OUT $_; } }
} elsif ($cmd eq 'stat') {
    print SOCK "$cmd\r\n";
    $answer = pop3_result (SOCK);
    pop_quit(SOCK, "protocol error!\n") unless (defined $answer);
    if ($answer) { print OUT "$answer\n"; }
} else {
    pop_quit(SOCK, "unknown or unimplemented command `$cmd'!\n");
}

unless ($answer) { pop_quit(SOCK, "POP3 command `$cmd' failed...\n"); }

print SOCK "QUIT\r\n";
unless (pop3_answer (SOCK)) {
    print STDERR "$progname(POP3): QUIT cmd failed - i will nonetheless ",
		 "close\n$filler        the connection.\n",
}
close SOCK;
close OUT;

exit 0;

sub pop3_answer (*) {
    my $SOCK = shift;
    local $_;
    $_ = <$SOCK>;
    (/^\+/ ? 1 : (/^\-/ ? 0 : undef))
}

sub pop3_result (*) {
    my $SOCK = shift;
    local $_;
    $_ = <$SOCK>;
    s/\r\n$//;
    (/^\+/ ? (substr $_, 4) : (/^\-/ ? '' : undef))
}

sub pwdprompt () {
    my ($pass, $_SIS);
    local $_;
    $_SIS = $|;
    $| = 1; print "Please enter the MAILSYS-password: ";
    system "stty", "-echo", "-icanon", "eof", "^@";
    $pass = '';
    for (;;) {
       	$_ = getc; last if ($_ eq "\n" or $_ eq "\r");
	if ($_ eq "\177" or $_ eq "\10") {
	    if ((length $pass) > 0) {
		$pass = substr $pass, 0, (length $pass) - 1; print "\10 \10";
	    }
	} else {
	    $pass .= $_; print "*";
	}
    }
    system "stty", "echo", "icanon", "eof", "^D";
    print "\n";
    $| = $_SIS;
    $pass
}

sub md5digest ($) {
    local $_ = `echo -n '$_[0]' | md5sum`;
    chomp;
    $_
}

sub pop3_ts (*) {
    my $SOCK = (shift);
    local $_;
    $_ = <$SOCK>;
    s/\r\n$//;
    if (/^\+.*(<.+>)/) {
	$1
    } else {
	(/^\+/ ? (substr $_, 4) : (/^\-/ ? '' : undef))
    }
}

sub pop3_status ($) {
    local $_ = (shift);
    s/^\s+//; s/\s+$//;
    split /\s+/, $_
}

sub pop_quit (*$) {
    my ($SOCK, $msg) = @_;
    print $SOCK "QUIT\r\n";
    close $SOCK;
    die "$progname(POP3): $msg\n";
}

sub usage (@) {
    my $li;
    if (@_) {
	$li = (shift);
	print STDERR "$progname: $li\n";
	for $li (@_) { print STDERR "$filler  $li\n"; }
    }
    die "Usage: $progname [-q] [-p <portnum>] <server> <POP3-cmd> <user> <POP3-args>\n",
	"\n  Options:",
	"\n    -q (alt: --quiet, -s, --silent)",
	"\n        turn off verbose output (more than one `-q' makes",
	"\n        `$progname' even more silent)",
	"\n    -p (alt: --port)",
	"\n        use <portnum> (instead of the default)\n",
	"\n  Arguments:",
	"\n    <server>",
	"\n        select the POP3-server",
	"\n    <POP3-cmd>",
	"          a POP3 command to be sent to the server",
	"\n    <user>",
	"\n        the selected user",
	"\n    <POP3-args>",
	"\n        additional arguments to be sent with <POP3-cmd>\n";
}
