/* digest_impl.c
**
** Implementation of the (exported) functions of the message digest algorithm
** collection.
**
*/

#include <sys/types.h>
#include "mrmacs.h"
#include "tools.h"

#include "digest-internal.h"
#include "dgTypeSel.h"

DIG_CTX digest_new (char *dgtype)
{
    int ix;
    DIG_CTX res = (DIG_CTX) galloc (__FILE__, __LINE__,
				    sizeof(struct _DIG_CTX));
    res->dgtype = -1;
    ifnull (dgtype) { res->dgtype = -1; res->dgc = NULL; return res; }
    for (ix = 0; noNULL(dgtrans[ix].dgNAME); ++ix) {
	if (!lccmp (dgtrans[ix].dgNAME, dgtype, -1)) {
	    res->dgtype = ix; break;
	}
    }
    if (res->dgtype < 0) { gfree (res); return (DIG_CTX) NULL; }
    res->dgc = galloc (__FILE__, __LINE__, dgtrans[ix].ctxsz);
    dgtrans[ix].init (res->dgc);
    return res;
}

void digest_free (DIG_CTX ctx)
{
    ifnull (ctx) { return; }
    unlessnull (ctx->dgc) { gfree (ctx->dgc); }
    gfree (ctx);
}

int digest_init (DIG_CTX ctx)
{
    int dgtype;
    ifnull (ctx) { return -1; }
    ifnull (ctx->dgc) { return -2; }
    dgtype = ctx->dgtype;
    dgtrans[dgtype].init (ctx->dgc);
    return 0;
}

int digest_update (DIG_CTX ctx, unsigned char *msg, size_t msglen)
{
    int dgtype;
    ifnull (ctx) { return -1; }
    ifnull (ctx->dgc) { return -2; }
    dgtype = ctx->dgtype;
    dgtrans[dgtype].update (ctx->dgc, msg, msglen);
    return 0;
}

int digest_final (unsigned char buf[], DIG_CTX ctx)
{
    int dgtype;
    ifnull (ctx) { return -1; }
    ifnull (ctx->dgc) { return -2; }
    dgtype = ctx->dgtype;
    dgtrans[dgtype].final (buf, ctx->dgc);
    return 0;
}

int digest_chgtype (DIG_CTX ctx, char *newtype)
{
    int ix, typex = -1;
    ifnull (ctx) { return -1; }
    for (ix = 0; noNULL(dgtrans[ix].dgNAME); ++ix) {
	if (!lccmp (newtype, dgtrans[ix].dgNAME, -1)) {
	    typex = ix; break;
	}
    }
    if (typex < 0) { return -2; }
    unlessnull (ctx->dgc) { gfree (ctx->dgc); }
    ctx->dgtype = typex;
    ctx->dgc = galloc (__FILE__, __LINE__, dgtrans[ix].ctxsz);
    dgtrans[ix].init (ctx->dgc);
    return 0;
}

ssize_t digest_size (DIG_CTX ctx)
{
    int dgtype;
    ifnull (ctx) { return -1; }
    ifnull (ctx->dgc) { return -2; }
    dgtype = ctx->dgtype;
    return dgtrans[dgtype].digsz;
}
