/* folder.c
**
** 2nd Level `xFile' access.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>

#include "xfile.h"
#include "tools.h"
#include "mrmacs.h"
#include "bcv.h"

/* Die Lnge der Prefix-Daten. Im Prefix wird der aktuelle Zhlerstand fr
** die `unique id' aufbewahrt.
*/
#define PFXSIZE (sizeof (time_t) + sizeof (int))

/* Die Lnge der Indexdaten. Hier steht (in diesem Fall) nur die `unique id'
** der E-Mail.
*/
#define XTRSIZE (sizeof (time_t) + sizeof (int))

/* Hilfsfunktionen zum Schreiben/lesen eines ganzzahligen Wertes
** in einen/aus einem Puffer.
*/
DEFCVF(b_to_int, int_to_b, int)

/* Hilfsfunktionen zum Schreiben/lesen eines Zeitwertes
** in einen/aus einem Puffer.
*/
DEFCVF(b_to_time, time_to_b, time_t)

/* Die Behandlung der Prefix- und Indexdaten wird hier versteckt. Wir
** haben schlielich den UID-Zhler als Prefix-Daten und die jeweilige UID
** als Indexdaten. Diese mssen an dieser Stelle auch nicht auftauchen.
*/
xFile flOpen (char *xname, char *amode, ...)
{
    va_list ap;
    char *nmode;
    int mode, uidlow = 0;
    time_t uidhigh = time ((time_t *) NULL);
    char buf[PFXSIZE];
    xFile res;
    nmode = gconc (__FILE__, __LINE__, amode, "e", NULL);
    unlessnull (strchr ("iuw", *amode)) {
	va_start (ap, amode);
	mode = va_arg (ap, int);
	va_end (ap);
	res = xOpen (xname, nmode, PFXSIZE, XTRSIZE, mode);
	if (res != (xFile) 0 && (xIsNew (res) || *amode == 'i')) {
	    time_to_b (uidhigh, buf);
	    int_to_b (uidlow, buf + sizeof (time_t));
	    xSetPrefix (res, buf, PFXSIZE);
	}
    } else {
	res = xOpen (xname, nmode, PFXSIZE, XTRSIZE);
    }
    gfree (nmode);
    return res;
}

/* Kopiere die UID der aktuellen E-Mail von einem Folder in den anderen. */
void flCopyUID (xFile fx, xFile fy)
{
    xCopyExtra (fx, fy, XTRSIZE);
}

/* Kopiere den aktuellen UID-Zhler des einen Folders in den anderen. */
void flCopyNextUID (xFile fx, xFile fy)
{
    xCopyPrefix (fx, fy, PFXSIZE);
}

/* Lies die UID der aktuellen E-Mail. */
void flGetUID (xFile fx, time_t *uidhigh, int *uidlow)
{
    char buf[XTRSIZE];
    xGetExtra (fx, buf, XTRSIZE);
    *uidhigh = b_to_time (buf);
    *uidlow = b_to_int (buf + sizeof (time_t));
}

/* Trage eine neue UID ein. */
void flSetNextUID (xFile fx)
{
    char pfx[PFXSIZE], xtr[XTRSIZE];
    time_t uidhigh;
    int uidlow;
    xGetPrefix (fx, pfx, PFXSIZE);
    uidhigh = b_to_time (pfx);
    uidlow = b_to_int (pfx + sizeof (time_t));
    xGetExtra (fx, xtr, XTRSIZE);
    time_to_b (uidhigh, xtr);
    int_to_b (uidlow, xtr + sizeof (time_t));
    ++uidlow;
    int_to_b (uidlow, pfx + sizeof (time_t));
    xSetExtra (fx, xtr, XTRSIZE);
    xSetPrefix (fx, pfx, PFXSIZE);
}
