/* tools.h
**
** Interface for file `tools.c'
** (Einige ntzliche Routinen)
*/
#ifndef _TOOLS_H
#define _TOOLS_H

#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <stdarg.h>

typedef void (*exception_t) (char *msg);

exception_t setup_exception(exception_t handle);

void heap_init(void);

char *sdupl(const char *string);

char *gdupl(char *file, int line, const char *string);

void *galloc(char *file, int line, size_t size);

void gfree(void *ref);

void *gresize(char *file, int line, void *ref, size_t size);

/* concat a NULL-terminated table of strings to a new string which is the
** result of this function. The result is NULL if there was insufficient memory
** for the result.
*/
char *vconcat(char **ap);

/* concat a NULL-terminated table of strings to a new string which is the
** result of this function. The result is NULL if there was insufficient memory
** for the result.
*/
char *concat(char *_1st, ...);

/* concat a NULL-terminated table of strings to a new string which is the
** result of this function. This function aborts if there was insufficient
** memory for the result.
*/
char *gconc(char *file, int line, ...);

void gappend (char *file, int line, char **res, ...);

void vxlog(int ll, char *fmt, ...);

void npfxlog(char *fmt, ...);

void vvxlog(int ll, char *fmt, va_list av);

void vxsetlog(char *logname);

void vxerr2log(void);

void vxerr2null(void);

int lccmp(const char *s1, const char *s2, ssize_t n);

void lcconv(char *bf, int bfl);

char *gvarins(char *file, int line, char *in, char *varname, const char *value);

int spawn (char *prog, ...);

mode_t reset_umask (void);

char *which (char *prog);

char *iftype (void);

FILE *ug_fopen (const char *path, const char *mode,
		const uid_t uid, const gid_t gid);

void memdump (int ll, char *buf, int len);

#endif /* defined(_TOOLS_H) */
